//****************************************************************************
//	
//	Copyright 2018 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : poll.cpp
//
//	Project	 : PCM-MIO Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	03/09/18	  1.0		Original Release
//
//****************************************************************************

#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <strsafe.h>
#include "pcmmioDLL.h"

DWORD WINAPI DioWaitForIrq(LPVOID lpParam);

#define MAJOR_VER   1
#define MINOR_VER   0
#define DEVICE      1

volatile int irq_count = 0;
volatile int exit_flag = 0;
char line[80];

int _tmain(int argc, _TCHAR* argv[])
{
    HANDLE hThread;
    DWORD dwThreadId;
    int ch;
    unsigned int portState[6] = { 0x00, 0x00, 0x00, 0xff, 0xff, 0xff };  // port1 + port2 + port3 inputs
    unsigned long timeout = 0x1000; // 4096 msec
    int dllReturn = SUCCESS;

    printf("PCM-MIO Application : poll\n");
    printf("Version %d.%d\n\n", MAJOR_VER, MINOR_VER);

    dllReturn = InitializeSession(DEVICE);

    if (dllReturn)
    {
        printf("Error initializing session.\n");
        exit(dllReturn);
    }
    else
    {
        printf("PCM-MIO device opened.\n");
    }

    dllReturn = DioSetIoMask(DEVICE, portState);

    if (dllReturn)
    {
        printf("Error setting I/O mask.\n");
        exit(dllReturn);
    }

    // enable rising edge interrupts for odd bits
    // enable falling edge interrupts for even bits
    for (int i = 0; i < 24; i++) {
        if (i % 2)
            dllReturn = DioEnableInterrupt(DEVICE, i, RISING_EDGE);
        else
            dllReturn = DioEnableInterrupt(DEVICE, i, FALLING_EDGE);

        if (dllReturn)
        {
            printf("Error enabling interrupts for bit %d.\n", i);
            exit(dllReturn);
        }
    }

    // Create the thread to begin execution on its own.
    hThread = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        DioWaitForIrq,          // thread function name
        &timeout,               // argument to thread function
        0,                      // use default creation flags 
        &dwThreadId);           // returns the thread identifier 

    // if CreateThread fails, terminate execution 
    if (hThread == NULL)
    {
        printf("Cannot create thread!\n");
        exit(1);
    }

    // print menu
    printf("Toggle bits to generate interrupts\n");
    printf("Odd bits configured for rising edge interrutps\n");
    printf("Even bits configured for falling edge interrutps\n");
    printf("Hit any key when done\n\n");

    // wait for key stroke
    while (!_kbhit()) ;
    ch = _getch();

    // done so set flag
    exit_flag = 1;
            
    // disable interrupts
    for (int i = 0; i < 24; i++) 
    {
        dllReturn = DioDisableInterrupt(DEVICE, i);

        if (dllReturn)
        {
            printf("Error disabling interrupt on bit %d.\n", i);
            exit(dllReturn);
        }
    }

    // Wait thread terminates
    WaitForSingleObject(hThread, INFINITE);

    // close thread handle
    CloseHandle(hThread);

    // finish
    dllReturn = CloseSession(DEVICE);

    if (dllReturn)
    {
        printf("Unable to close device.\n");
        exit(dllReturn);
    }
    else
        printf("PCM-MIO device closed.\n");

    // display our event count total
    printf("\nInterrupt count = %d\r", irq_count);
}

DWORD WINAPI DioWaitForIrq(LPVOID lpParam)
{
    BYTE irqBit = 0;
    unsigned int irqArray[3];
    int dllReturn = SUCCESS;
    int count = 0;
    unsigned long *timeout = (unsigned long*) lpParam;

    while (1)
    {
        // put THIS process to sleep until interrupt occurs
        dllReturn = DioWaitForInterrupt(DEVICE, irqArray, *timeout);

        // exit loop if program is terminating
        if (exit_flag)
            break;

        // restart loop if error
        if (dllReturn)
            continue;

        // determine interrupt bit
        if (irqArray[0]) {
            while (!(irqArray[0] & (1 << count)))
                count++;

            irqBit = count + 1;
        }
        else if (irqArray[1]) {
            while (!(irqArray[1] & (1 << count)))
                count++;

            irqBit = count + 9;
        }
        else if (irqArray[2]) {
            while (!(irqArray[2] & (1 << count)))
                count++;

            irqBit = count + 17;
        }

        if (irqBit)
            printf("Interrupt on bit %d\n", irqBit);
        else
            printf("No Interrupt\n");

        count = 0;
        irq_count++;
    }

    return 0;
}
