//****************************************************************************
//	
//	Copyright 2018 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : setvolt_irq.cpp
//
//	Project	 : PCM-MIO Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	03/09/18	  1.0		Original Release
//
//****************************************************************************

#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <strsafe.h>
#include "pcmmioDLL.h"

DWORD WINAPI DacConversion1(LPVOID lpParam);
DWORD WINAPI DacConversion2(LPVOID lpParam);

#define MAJOR_VER    1
#define MINOR_VER    0
#define DEVICE       1
#define DAC1_CHANNEL 2
#define DAC2_CHANNEL 5

volatile int conv_count1 = 0;
volatile int conv_count2 = 0;
volatile int exit_flag = 0;

int _tmain(int argc, _TCHAR* argv[])
{
    HANDLE hThread[2];
    DWORD dwThreadId[2];
    unsigned long timeout = 0x1000; // 4096 msec
    int dllReturn = SUCCESS;

    printf("PCM-MIO Application : setvolt_irq\n");
    printf("Version %d.%d\n\n", MAJOR_VER, MINOR_VER);

    dllReturn = InitializeSession(DEVICE);

    if (dllReturn)
    {
        printf("Error initializing session.\n");
        exit(dllReturn);
    }
    else
    {
        printf("PCM-MIO device opened.\n");
    }

    // set all dac channels to bipolar -10v to 10v
    for (int ch = 0; ch < 8; ch++) {
        dllReturn = DacSetChannelSpan(DEVICE, ch, DAC_SPAN_BI10);

        if (dllReturn)
        {
            printf("Error setting DAC channel %d span.\n", ch);
            return 1;
        }
        else
        {
            printf("DAC Channel %d span set to Bipolar -10V to 10V.\n", ch);
        }
    }
    // enable interrupts for dac1 and dac2
    dllReturn = DacEnableInterrupt(DEVICE, DAC1_CHANNEL); // channel 0 will enable dac1 interrupts

    if (dllReturn)
    {
        printf("Error enabling interrupts for DAC1.\n");
        exit(dllReturn);
    }

    dllReturn = DacEnableInterrupt(DEVICE, DAC2_CHANNEL); // channel 4 will enable dac2 interrupts

    if (dllReturn)
    {
        printf("Error enabling interrupts for DAC2.\n");
        exit(dllReturn);
    }

    // Create threads to begin execution
    hThread[0] = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        DacConversion1,         // thread function name
        &timeout,               // argument to thread function
        0,                      // use default creation flags 
        &dwThreadId[0]);        // returns the thread identifier 

    // if CreateThread fails, terminate execution 
    if (hThread[0] == NULL)
    {
        printf("Cannot create thread!\n");
        exit(1);
    }

    hThread[1] = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        DacConversion2,         // thread function name
        &timeout,               // argument to thread function
        0,                      // use default creation flags 
        &dwThreadId[1]);        // returns the thread identifier 

    // if CreateThread fails, terminate execution 
    if (hThread[1] == NULL)
    {
        printf("Cannot create thread!\n");
        exit(1);
    }

    // print heartbeat
    do {
        printf(" * ");

        Sleep(10);
    } while (!_kbhit()); // wait for key stroke

    int ch = _getch();

    // done so set flag
    exit_flag = 1;
            
    // disable interrupts for dac1 and dac2
    dllReturn = DacDisableInterrupt(DEVICE, DAC1_CHANNEL); // channel 0 will disable dac1 interrupts

    if (dllReturn)
    {
        printf("Error disabling interrupts for DAC1.\n");
        exit(dllReturn);
    }

    dllReturn = DacDisableInterrupt(DEVICE, DAC2_CHANNEL); // channel 4 will disable dac2 interrupts

    if (dllReturn)
    {
        printf("Error disabling interrupts for DAC2.\n");
        exit(dllReturn);
    }

    // Wait for threads to terminate
    WaitForSingleObject(hThread[0], INFINITE);
    WaitForSingleObject(hThread[1], INFINITE);

    // close thread handles
    CloseHandle(hThread[0]);
    CloseHandle(hThread[1]);

    // finish
    dllReturn = CloseSession(DEVICE);

    if (dllReturn)
    {
        printf("Unable to close device.\n");
        exit(dllReturn);
    }
    else
        printf("\nPCM-MIO device closed.\n");

    // display our event count total
    printf("\nConversion count DAC1 = %d\r", conv_count1);
    printf("\nConversion count DAC2 = %d\r", conv_count2);
}

DWORD WINAPI DacConversion1(LPVOID lpParam)
{
    unsigned short dacCode = 0xc000;
    unsigned long *timeout = (unsigned long*) lpParam;
    int dllReturn = SUCCESS;

    // use interrupts now
    while (!exit_flag)
    {
        // put THIS process to sleep until update completes
        dllReturn = DacWaitForUpdate(DEVICE, DAC1_CHANNEL, dacCode, *timeout);

        // restart loop if error
        if (dllReturn) {
            printf("Error %d ...\n", dllReturn);
            continue;
        }
        else {
            printf("\nDAC Channel %d value set to %04x.\n", DAC1_CHANNEL, dacCode);

            conv_count1++;
        }

        dacCode += 0x80;

        Sleep(50);
    }

    return dllReturn;
}

DWORD WINAPI DacConversion2(LPVOID lpParam)
{
    unsigned short dacCode = 0x4000;
    unsigned long *timeout = (unsigned long*)lpParam;
    int dllReturn = SUCCESS;

    // use interrupts now
    while (!exit_flag)
    {
        // put THIS process to sleep until update completes
        dllReturn = DacWaitForUpdate(DEVICE, DAC2_CHANNEL, dacCode, *timeout);

        // restart loop if error
        if (dllReturn) {
            printf("Error %d ...\n", dllReturn);
            continue;
        }
        else {
            printf("\nDAC Channel %d value set to %04x.\n", DAC2_CHANNEL, dacCode);

            conv_count2++;
        }

        dacCode += 0x40;

        Sleep(100);
    }

    return dllReturn;
}
